package com.bytesw;

import com.bytesw.juego.Crupier;
import com.bytesw.juego.Juego;
import com.bytesw.juego.Jugador;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;
import java.io.ByteArrayOutputStream;
import java.io.PrintStream;
import java.util.Scanner;
import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class BlackJackTest {

    private ByteArrayOutputStream out;

    @InjectMocks
    private Juego main = new Juego();

    @Mock
    Scanner scanner;
    @Mock
    Jugador jugador;
    @Mock
    Crupier crupier;

    @Before
    public void setup() {
        out = new ByteArrayOutputStream();
        System.setOut(new PrintStream(out));
    }

    @Test
    public void cuandoEscribeNombreEntoncesPreguntaSiQuiereJugar(){
        when(scanner.nextLine()).thenReturn("Charles");
        main.jugar();
        Assert.assertTrue(out.toString().contains("Si quieres empezar a jugar "+
                                                    "escribe: jugar"));
    }

    @Test
    public void testCuandoIngresaSaldo0MuestraSaldoNoSuficiente(){
        when(scanner.nextLine()).thenReturn("Charles").thenReturn("jugar").
                thenReturn("0");
        main.jugar();
        Assert.assertTrue(out.toString().contains("SALDO NO SUFICIENTE"));
    }

    @Test
    public void testCuandoEscribeSaldoMuestraElSaldoDisponible(){
        when(scanner.nextLine()).thenReturn("Charles").thenReturn("jugar").
                        thenReturn("200").thenReturn("saldo").thenReturn("no");
        main.jugar();
        Assert.assertTrue(out.toString().contains("Tu saldo es: 200"));
    }


    @Test
    public void testCuandoEscribeHistoricoMuestraLosHistoricos(){
        when(scanner.nextLine()).thenReturn("Charles").thenReturn("jugar").
                thenReturn("200").thenReturn("historico").thenReturn("no");
        main.jugar();
        Assert.assertTrue(out.toString().contains("Mostrando historico de " +
                                                  "jugadas..."));
    }

    @Test
    public void testCuandoEscribeNivelesMuestraLosNivelesDisponibles(){
        when(scanner.nextLine()).thenReturn("Charles").thenReturn("jugar").
                thenReturn("200").thenReturn("niveles");
        main.jugar();
        Assert.assertTrue(out.toString().contains("***** Niveles Disponibles " +
                                                  "*****"));
    }

    @Test
    public void testCuandoIngresaAlNivel2MuestraBlackJack(){
        when(scanner.nextLine()).thenReturn("Charles").thenReturn("jugar").
                thenReturn("200").thenReturn("niveles").thenReturn("Nivel 2");
        main.jugar();
        Assert.assertTrue(out.toString().contains("*****BLACKJACK*****"));
    }

    @Test
    public void testCuandoIngresaAlNivel2PideLaApuesta(){
        when(scanner.nextLine()).thenReturn("Charles").thenReturn("jugar").
                thenReturn("200").thenReturn("niveles").thenReturn("Nivel 2");
        main.jugar();
        Assert.assertTrue(out.toString().contains("Para empezar ingrese su " +
                                                    "apuesta"));
    }

    @Test
    public void testGanaJugador(){
        when(jugador.obtenerValorDeLaMano()).thenReturn(18);
        when(crupier.obtenerValorDeLaMano()).thenReturn(11);
        assertEquals("Gano el Jugador!",main.encontrarGanadorBlackJack());
    }

    @Test
    public void testGanaCrupier(){
        when(jugador.obtenerValorDeLaMano()).thenReturn(18);
        when(crupier.obtenerValorDeLaMano()).thenReturn(20);
        assertEquals("Gano el Crupier!",main.encontrarGanadorBlackJack());
    }

    @Test
    public void testEmpatan(){
        when(jugador.obtenerValorDeLaMano()).thenReturn(20);
        when(crupier.obtenerValorDeLaMano()).thenReturn(20);
        assertEquals("Empataron nadie gano",main.encontrarGanadorBlackJack());
    }

    @Test
    public void testEmpatanCon21(){
        when(jugador.obtenerValorDeLaMano()).thenReturn(21);
        when(crupier.obtenerValorDeLaMano()).thenReturn(21);
        assertEquals("Empataron nadie gano",main.encontrarGanadorBlackJack());
    }

    @Test
    public void testJugadorConsigueBlackJack(){
        when(jugador.getJugadas()).thenReturn(2);
        when(jugador.obtenerValorDeLaMano()).thenReturn(21);
        when(crupier.obtenerValorDeLaMano()).thenReturn(5);
        assertEquals("BlackJack --> Paga por 3",main.encontrarGanadorBlackJack());
    }

    @Test
    public void testJugadorConsigueBlackJackPeroEmpata(){
        when(jugador.getJugadas()).thenReturn(2);
        when(jugador.obtenerValorDeLaMano()).thenReturn(21);
        when(crupier.obtenerValorDeLaMano()).thenReturn(21);
        assertEquals("Empataron nadie gano",main.encontrarGanadorBlackJack());
    }

    @Test
    public void testPierdeElCrupierYGanaElJugador(){
        when(jugador.obtenerValorDeLaMano()).thenReturn(12);
        when(crupier.obtenerValorDeLaMano()).thenReturn(23);
        assertEquals("Gano el Jugador!",main.encontrarGanadorBlackJack());
    }
    @Test
    public void testPierdeElJuadorYGanaElCrupier(){
        when(jugador.obtenerValorDeLaMano()).thenReturn(25);
        when(crupier.obtenerValorDeLaMano()).thenReturn(20);
        assertEquals("Gano el Crupier!",main.encontrarGanadorBlackJack());
    }

    //Pierden los dos NO PUEDE PASAR
    @Test
    public void testPierdenLosDos(){
        when(jugador.obtenerValorDeLaMano()).thenReturn(23);
        when(crupier.obtenerValorDeLaMano()).thenReturn(28);
        assertEquals("Perdieron los dos",main.encontrarGanadorBlackJack());
    }

}
